@@ -12,7 +12,8 @@ from inspect import isclass
 import cherrypy
 from cherrypy import request, response
 
-from peak.rules import abstract, always_overrides, Method, NoApplicableMethods
+from peak.rules import abstract, NoApplicableMethods
+from peak.rules.core import always_overrides, Method
 
 import turbogears.util as tg_util
 from turbogears import view, database, errorhandling, config
@@ -46,25 +47,30 @@ def _process_output(output, template, format, content_type, fragment=False,
         css = tg_util.setlike()
         js = dict(izip(js_location, iter(tg_util.setlike, None)))
         include_widgets = {}
-        include_widgets_lst = config.get("tg.include_widgets", [])
+        include_widgets_lst = config.get('tg.include_widgets', [])
 
-        if config.get("tg.mochikit_all", False):
+        if config.get('tg.mochikit_all', False):
             include_widgets_lst.insert(0, 'turbogears.mochikit')
 
         for name in include_widgets_lst:
             widget = tg_util.load_class(name)
+            if widget is None:
+                log.debug("Could not load widget %s", name)
+                continue
             if isclass(widget):
                 widget = widget()
+            if hasattr(widget, 'retrieve_resources') and hasattr(widget, 'inject'):
+                # it's a ToscaWidget, we register it for injection
+                widget.inject()
             # XXX: widgets with same base name will override each other
-            include_widgets["tg_%s" % name.split(".")[-1]] = widget
+            include_widgets['tg_%s' % name.rsplit('.', 1)[-1]] = widget
         output.update(include_widgets)
 
         # collect JS/CSS resources from widgets in the output dict or
         # tg.include_widgets
         for value in output.itervalues():
-            if hasattr(value, "retrieve_resources"):
-                # it's a tw widget --> ToscaWidgets takes care of JS/CSS
-                # resource inclusion
+            if hasattr(value, 'retrieve_resources'):
+                # it's a ToscaWidget, will be injected by the ToscaWidget middleware
                 continue
             else:
                 try:
@@ -77,17 +83,17 @@ def _process_output(output, template, format, content_type, fragment=False,
                     js_resources = []
             css.add_all(css_resources)
             for script in js_resources:
-                location = getattr(script, "location", js_location.head)
+                location = getattr(script, 'location', js_location.head)
                 js[location].add(script)
-        css.sort(key=lambda obj:  getattr(obj, "order", 0))
-        output["tg_css"] = css
+        css.sort(key=lambda obj:  getattr(obj, 'order', 0))
+        output['tg_css'] = css
         for location in iter(js_location):
-            js[location].sort(key=lambda obj:  getattr(obj, "order", 0))
-            output["tg_js_%s" % location] = js[location]
+            js[location].sort(key=lambda obj:  getattr(obj, 'order', 0))
+            output['tg_js_%s' % location] = js[location]
 
         tg_flash = _get_flash()
         if tg_flash:
-            output["tg_flash"] = tg_flash
+            output['tg_flash'] = tg_flash
 
         headers = {'Content-Type': content_type}
         output = view.render(output, template=template, format=format,
@@ -95,11 +101,11 @@ def _process_output(output, template, format, content_type, fragment=False,
         content_type = headers['Content-Type']
 
     if content_type:
-        response.headers["Content-Type"] = content_type
+        response.headers['Content-Type'] = content_type
     else:
-        content_type = response.headers.get("Content-Type", 'text/plain')
+        content_type = response.headers.get('Content-Type', 'text/plain')
 
-    if content_type.startswith("text/"):
+    if content_type.startswith('text/'):
         if isinstance(output, unicode):
             output = output.encode(tg_util.get_template_encoding_default())
 
@@ -226,8 +232,8 @@ def _add_rule(_expose, found_default, as_format, accept_format, template,
             found_default = True
     ruleparts = ['kw.get("tg_format", "default") == "%s"' % as_format]
     if accept_format:
-        ruleparts.append('(accept == "%s" and kw.get("tg_format"'
-            ', "default") == "default")' % accept_format)
+        ruleparts.append('(accept == "%s" and '
+            'kw.get("tg_format", "default") == "default")' % accept_format)
     rule = " or ".join(ruleparts)
     log.debug("Generated rule %s", rule)
     first(_expose, rule)(rulefunc)
@@ -240,11 +246,10 @@ def _build_rules(func):
         pass
 
     if func._allow_json:
-        log.debug("Adding allow_json rule: "
-            'allow_json and (kw.get("tg_format", None) == "json"'
+        rule = ('allow_json and (kw.get("tg_format", None) == "json"'
             ' or accept in ("application/json", "text/javascript"))')
-        first(_expose, 'allow_json and (kw.get("tg_format", None) == "json"'
-            ' or accept in ("application/json", "text/javascript"))')(
+        log.debug("Adding allow_json rule for %s: %s", func, rule)
+        first(_expose, rule)(
             lambda _func, accept, allow_json, *args, **kw:
                 _execute_func(_func, "json", "json", "application/json",
                     False, {}, args, kw))
@@ -440,7 +445,7 @@ def _execute_func(func, template, format, content_type, fragment, options,
             format = output.pop("tg_format", format)
 
         if template and template.startswith("."):
-            template = func.__module__[:func.__module__.rfind('.')]+template
+            template = func.__module__[:func.__module__.rfind('.')] + template
 
         return _process_output(output, template, format, content_type,
                                fragment, **options)
@@ -448,7 +453,9 @@ def _execute_func(func, template, format, content_type, fragment, options,
 
 def flash(message):
     """Set a message to be displayed in the browser on next page display."""
-    message = tg_util.quote_cookie(tg_util.to_utf8(message))
+    message = tg_util.to_utf8(message)
+    if len(message) > 4000:
+        log.warning('Flash message exceeding maximum cookie size!')
     response.simple_cookie['tg_flash'] = message
     response.simple_cookie['tg_flash']['path'] = '/'
 
@@ -463,20 +470,20 @@ def _get_flash():
         response_cookie["tg_flash"]['expires'] = 0
         response_cookie['tg_flash']['path'] = '/'
 
-    if response_cookie.has_key("tg_flash"):
+    if "tg_flash" in response_cookie:
         message = response_cookie["tg_flash"].value
         response_cookie.pop("tg_flash")
-        if request_cookie.has_key("tg_flash"):
+        if "tg_flash" in request_cookie:
             # New flash overrided old one sitting in cookie. Clear that old cookie.
             clearcookie()
-    elif request_cookie.has_key("tg_flash"):
-        message = request_cookie["tg_flash"].value
-        if not response_cookie.has_key("tg_flash"):
+    elif "tg_flash" in request_cookie:
+        message = request_cookie.value_decode(request_cookie["tg_flash"].value)[0]
+        if "tg_flash" not in response_cookie:
             clearcookie()
     else:
         message = None
     if message:
-        message = unicode(tg_util.unquote_cookie(message), 'utf-8')
+        message = unicode(message, 'utf-8')
     return message
 
 
@@ -489,39 +496,34 @@ class Controller(object):
 
     """
 
-
-class RootController(Controller):
-    """Base class for the root of a web application.
-
-    Your web application must have one of these. The root of your application
-    is used to compute URLs used by your app.
-
-    """
-    is_app_root = True
-
     msglog = logging.getLogger('cherrypy.msg')
     msglogfunc = {0: msglog.info, 1: msglog.warning, 2: msglog.error}
-    def _cp_log_message(self, msg, context = 'nocontext', severity = 0):
-        log = self.msglogfunc[severity]
+
+    @classmethod
+    def _cp_log_message(cls, msg, context = 'nocontext', severity = 0):
+        """Default method for logging messages (errors and app-specific info)"""
+        log = cls.msglogfunc[severity]
         text = ''.join((context, ': ', msg))
         log(text)
 
-    accesslog = logging.getLogger("turbogears.access")
-    def _cp_log_access(self):
+    accesslog = logging.getLogger('turbogears.access')
+
+    @classmethod
+    def _cp_log_access(cls):
+        """Default method for logging access"""
         # Conforms to Combined Log Format
         # http://httpd.apache.org/docs/2.2/logs.html#combined
-        tmpl = '%(host)s %(ident)s %(authuser)s [%(date)s] "%(request)s" ' \
-            '%(status)s %(bytes)s "%(referrer)s" "%(useragent)s"'
+        tmpl = ('%(host)s %(ident)s %(authuser)s [%(date)s] "%(request)s"'
+            ' %(status)s %(bytes)s "%(referrer)s" "%(useragent)s"')
         try:
             username = request.user_name
-            if username:
-                if isinstance(username, unicode):
-                    # might get concatenated to non-ascii byte strings
-                    username = username.encode('utf-8')
+            if username and isinstance(username, unicode):
+                # might get concatenated to non-ascii byte strings
+                username = username.encode('utf-8')
             else:
-                username = "-"
+                username = '-'
         except AttributeError:
-            username = "-"
+            username = '-'
         request_date = time.strftime('%d/%b/%Y:%H:%M:%S +0000',time.gmtime())
         request_info = {
             'host': request.headers.get('X-Forwarded-For')
@@ -530,12 +532,23 @@ class RootController(Controller):
             'authuser': username,
             'date': request_date,
             'request': request.requestLine,
-            'status': response.status.split(" ", 1)[0],
-            'bytes': response.headers.get('Content-Length') or "-",
-            'referrer': request.headers.get('referer', ""),
-            'useragent': request.headers.get('user-agent', ""),
+            'status': response.status.split(None, 1)[0],
+            'bytes': response.headers.get('Content-Length') or '-',
+            'referrer': request.headers.get('referer', ''),
+            'useragent': request.headers.get('user-agent', ''),
         }
-        self.accesslog.info(tmpl, request_info)
+        cls.accesslog.info(tmpl, request_info)
+
+
+class RootController(Controller):
+    """Base class for the root of a web application.
+
+    Your web application must have one of these. The root of your application
+    is used to compute URLs used by your app.
+
+    """
+
+    is_app_root = True
 
 Root = RootController
 
@@ -556,7 +569,7 @@ class ExposedDescriptor(object):
             cp_methodname, cherrypy.request.browser_url))
 
 
-class RESTMethod(object):
+class RESTMethod(Controller):
     """Allow REST style dispatch based on different HTTP methods.
 
     For an elaborate usage example see turbogears.tests.test_restmethod.
@@ -579,6 +592,7 @@ class RESTMethod(object):
                     ...
 
     """
+
     exposed = ExposedDescriptor()
 
     def __init__(self, *l, **kw):
