@@ -1,16 +1,17 @@
-from sqlalchemy.orm import class_mapper
+import logging
 
 from turbogears import config, identity
-from turbogears.database import session
+from turbogears.database import bind_metadata, session
 from turbogears.util import load_class
 
-import logging
-log = logging.getLogger("turbogears.identity.saprovider")
-
+from sqlalchemy.orm import class_mapper
 try:
-    set, frozenset
-except NameError: # Python 2.3
-    from sets import Set as set, ImmutableSet as frozenset
+    from sqlalchemy.exc import IntegrityError
+except ImportError: # SQLAlchemy < 0.5
+    from sqlalchemy.exceptions import IntegrityError
+
+
+log = logging.getLogger("turbogears.identity.saprovider")
 
 
 # Global class references --
@@ -31,7 +32,8 @@ class SqlAlchemyIdentity(object):
             if visit_key is not None:
                 self.login()
 
-    def _get_user(self):
+    @property
+    def user(self):
         """Get user instance for this identity."""
         try:
             return self._user
@@ -41,30 +43,30 @@ class SqlAlchemyIdentity(object):
         # Attempt to load the user. After this code executes, there *will* be
         # a _user attribute, even if the value is None.
         visit = self.visit_link
-        self._user = visit and user_class.query.get(visit.user_id)
+        self._user = visit and session.query(user_class).get(visit.user_id)
         return self._user
-    user = property(_get_user)
 
-    def _get_user_name(self):
+    @property
+    def user_name(self):
         """Get user name of this identity."""
         if not self.user:
             return None
         return self.user.user_name
-    user_name = property(_get_user_name)
 
-    def _get_user_id(self):
+    @property
+    def user_id(self):
         """Get user id of this identity."""
         if not self.user:
             return None
         return self.user.user_id
-    user_id = property(_get_user_id)
 
-    def _get_anonymous(self):
+    @property
+    def anonymous(self):
         """Return true if not logged in."""
         return not self.user
-    anonymous = property(_get_anonymous)
 
-    def _get_permissions(self):
+    @property
+    def permissions(self):
         """Get set of permission names of this identity."""
         try:
             return self._permissions
@@ -75,11 +77,11 @@ class SqlAlchemyIdentity(object):
             self._permissions = frozenset()
         else:
             self._permissions = frozenset(
-                [p.permission_name for p in self.user.permissions])
+                p.permission_name for p in self.user.permissions)
         return self._permissions
-    permissions = property(_get_permissions)
 
-    def _get_groups(self):
+    @property
+    def groups(self):
         """Get set of group names of this identity."""
         try:
             return self._groups
@@ -89,11 +91,11 @@ class SqlAlchemyIdentity(object):
         if not self.user:
             self._groups = frozenset()
         else:
-            self._groups = frozenset([g.group_name for g in self.user.groups])
+            self._groups = frozenset(g.group_name for g in self.user.groups)
         return self._groups
-    groups = property(_get_groups)
 
-    def _get_group_ids(self):
+    @property
+    def group_ids(self):
         """Get set of group IDs of this identity."""
         try:
             return self._group_ids
@@ -103,32 +105,38 @@ class SqlAlchemyIdentity(object):
         if not self.user:
             self._group_ids = frozenset()
         else:
-            self._group_ids = frozenset([g.group_id for g in self.user.groups])
+            self._group_ids = frozenset(g.group_id for g in self.user.groups)
         return self._group_ids
-    group_ids = property(_get_group_ids)
 
-    def _get_visit_link(self):
+    @property
+    def visit_link(self):
         """Get the visit link to this identity."""
         if self.visit_key is None:
             return None
-        return visit_class.query.filter_by(visit_key=self.visit_key).first()
-    visit_link = property(_get_visit_link)
+        return session.query(visit_class).filter_by(
+            visit_key=self.visit_key).first()
 
-    def _get_login_url(self):
+    @property
+    def login_url(self):
         """Get the URL for the login page."""
         return identity.get_failure_url()
-    login_url = property(_get_login_url)
 
     def login(self):
         """Set the link between this identity and the visit."""
         visit = self.visit_link
-        if visit:
-            visit.user_id = self._user.user_id
-        else:
+        if not visit:
             visit = visit_class()
             visit.visit_key = self.visit_key
+            session.add(visit)
+        visit.user_id = self._user.user_id
+        try:
+            session.flush()
+        except IntegrityError:
+            visit = self.visit_link
+            if not visit:
+                raise
             visit.user_id = self._user.user_id
-        session.flush()
+            session.flush()
 
     def logout(self):
         """Remove the link between this identity and the visit."""
@@ -145,27 +153,29 @@ class SqlAlchemyIdentityProvider(object):
 
     def __init__(self):
         super(SqlAlchemyIdentityProvider, self).__init__()
-        get = config.get
+        glob_ns = globals()
 
-        global user_class, group_class, permission_class, visit_class
+        for classname in ('user', 'group', 'permission', 'visit'):
+            default_classname = '.TG_' + (classname == 'visit'
+                and 'VisitIdentity' or classname.capitalize())
+            class_path = config.get("identity.saprovider.model.%s" % classname,
+                __name__ + default_classname)
+            class_ = load_class(class_path)
+            if class_:
+                log.info('Successfully loaded "%s".', class_path)
+                glob_ns['%s_class' % classname] = class_
+            else:
+                log.error('Could not load class "%s". Check '
+                    'identity.saprovider.model.%s setting', class_path, classname)
 
-        user_class_path = get("identity.saprovider.model.user", None)
-        user_class = load_class(user_class_path)
-        group_class_path = get("identity.saprovider.model.group", None)
-        group_class = load_class(group_class_path)
-        permission_class_path = get(
-            "identity.saprovider.model.permission", None)
-        permission_class = load_class(permission_class_path)
-        visit_class_path = get("identity.saprovider.model.visit", None)
-        log.info("Loading: %s", visit_class_path)
-        visit_class = load_class(visit_class_path)
+    def encrypt_password(self, password):
         # Default encryption algorithm is to use plain text passwords
-        algorithm = get("identity.saprovider.encryption_algorithm", None)
-        self.encrypt_password = lambda pw: \
-            identity._encrypt_password(algorithm, pw)
+        algorithm = config.get("identity.saprovider.encryption_algorithm", None)
+        return identity.encrypt_pw_with_algorithm(algorithm, password)
 
     def create_provider_model(self):
         """Create the database tables if they don't already exist."""
+        bind_metadata()
         class_mapper(user_class).local_table.create(checkfirst=True)
         class_mapper(group_class).local_table.create(checkfirst=True)
         class_mapper(permission_class).local_table.create(checkfirst=True)
@@ -182,15 +192,19 @@ class SqlAlchemyIdentityProvider(object):
             permissions: a set of permission names
 
         """
-        user = user_class.query.filter_by(user_name=user_name).first()
+        user = session.query(user_class).filter_by(
+                user_name=user_name).first()
+
         if not user:
             log.warning("No such user: %s", user_name)
             return None
+
         if not self.validate_password(user, user_name, password):
             log.info("Passwords don't match for user: %s", user_name)
             return None
+
         log.info("Associating user (%s) with visit (%s)",
-            user_name, visit_key)
+                user_name, visit_key)
         return SqlAlchemyIdentity(visit_key, user)
 
     def validate_password(self, user, user_name, password):
