@@ -1,27 +1,21 @@
+import logging
 from datetime import datetime
 
-from sqlobject import SQLObject, SQLObjectNotFound, RelatedJoin, \
-    DateTimeCol, IntCol, StringCol, UnicodeCol
-from sqlobject.inheritance import InheritableSQLObject
-
-import warnings
-import logging
-log = logging.getLogger("turbogears.identity.soprovider")
-
-import turbogears
-from turbogears import identity
+from turbogears import config, identity
 from turbogears.database import PackageHub
 from turbogears.util import load_class
 from turbojson.jsonify import jsonify_sqlobject, jsonify
 
+from sqlobject import (SQLObject, SQLObjectNotFound, RelatedJoin,
+    DateTimeCol, IntCol, StringCol, UnicodeCol)
+from sqlobject.dberrors import DuplicateEntryError
+
+
+log = logging.getLogger("turbogears.identity.soprovider")
+
 hub = PackageHub("turbogears.identity")
 __connection__ = hub
 
-try:
-    set, frozenset
-except NameError:  # Python 2.3
-    from sets import Set as set, ImmutableSet as frozenset
-
 
 def to_db_encoding(s, encoding):
     if isinstance(s, str):
@@ -33,22 +27,6 @@ def to_db_encoding(s, encoding):
     return s
 
 
-class DeprecatedAttr(object):
-    def __init__(self, old_name, new_name):
-        self.old_name = old_name
-        self.new_name = new_name
-
-    def __get__(self, obj, type=None):
-        warnings.warn("%s has been deprecated in favor of %s" %
-            (self.old_name, self.new_name), DeprecationWarning)
-        return getattr(obj, self.new_name)
-
-    def __set__(self, obj, value):
-        warnings.warn( "%s has been deprecated in favor of %s" %
-            (self.old_name, self.new_name), DeprecationWarning)
-        return setattr(obj, self.new_name, value)
-
-
 # Global class references --
 # these will be set when the provider is initialized.
 user_class = None
@@ -67,7 +45,8 @@ class SqlObjectIdentity(object):
             if visit_key is not None:
                 self.login()
 
-    def _get_user(self):
+    @property
+    def user(self):
         """Get user instance for this identity."""
         try:
             return self._user
@@ -86,28 +65,28 @@ class SqlObjectIdentity(object):
         else:
             self._user = None
         return self._user
-    user = property(_get_user)
 
-    def _get_user_name(self):
+    @property
+    def user_name(self):
         """Get user name of this identity."""
         if not self.user:
             return None
         return self.user.user_name
-    user_name = property(_get_user_name)
 
-    def _get_user_id(self):
+    @property
+    def user_id(self):
         """Get user id of this identity."""
         if not self.user:
             return None
         return self.user.id
-    user_id = property(_get_user_id)
 
-    def _get_anonymous(self):
+    @property
+    def anonymous(self):
         """Return true if not logged in."""
         return not self.user
-    anonymous = property(_get_anonymous)
 
-    def _get_permissions(self):
+    @property
+    def permissions(self):
         """Get set of permission names of this identity."""
         try:
             return self._permissions
@@ -118,11 +97,11 @@ class SqlObjectIdentity(object):
             self._permissions = frozenset()
         else:
             self._permissions = frozenset(
-                [p.permission_name for p in self.user.permissions])
+                p.permission_name for p in self.user.permissions)
         return self._permissions
-    permissions = property(_get_permissions)
 
-    def _get_groups(self):
+    @property
+    def groups(self):
         """Get set of group names of this identity."""
         try:
             return self._groups
@@ -132,11 +111,11 @@ class SqlObjectIdentity(object):
         if not self.user:
             self._groups = frozenset()
         else:
-            self._groups = frozenset([g.group_name for g in self.user.groups])
+            self._groups = frozenset(g.group_name for g in self.user.groups)
         return self._groups
-    groups = property(_get_groups)
 
-    def _get_group_ids(self):
+    @property
+    def group_ids(self):
         """Get set of group IDs of this identity."""
         try:
             return self._group_ids
@@ -146,11 +125,11 @@ class SqlObjectIdentity(object):
         if not self.user:
             self._group_ids = frozenset()
         else:
-            self._group_ids = frozenset([g.id for g in self.user.groups])
+            self._group_ids = frozenset(g.id for g in self.user.groups)
         return self._group_ids
-    group_ids = property(_get_group_ids)
 
-    def _get_visit_link(self):
+    @property
+    def visit_link(self):
         """Get the visit link to this identity."""
         if self.visit_key is None:
             return None
@@ -158,12 +137,11 @@ class SqlObjectIdentity(object):
             return visit_class.by_visit_key(self.visit_key)
         except SQLObjectNotFound:
             return None
-    visit_link = property(_get_visit_link)
 
-    def _get_login_url(self):
+    @property
+    def login_url(self):
         """Get the URL for the login page."""
         return identity.get_failure_url()
-    login_url = property(_get_login_url)
 
     def login(self):
         """Set the link between this identity and the visit."""
@@ -171,7 +149,14 @@ class SqlObjectIdentity(object):
         if visit:
             visit.user_id = self._user.id
         else:
-            visit = visit_class(visit_key=self.visit_key, user_id=self._user.id)
+            try:
+                visit = visit_class(
+                    visit_key=self.visit_key, user_id=self._user.id)
+            except DuplicateEntryError:
+                visit = self.visit_link
+                if not visit:
+                    raise
+                visit.user_id = self._user.id
 
     def logout(self):
         """Remove the link between this identity and the visit."""
@@ -187,42 +172,30 @@ class SqlObjectIdentityProvider(object):
 
     def __init__(self):
         super(SqlObjectIdentityProvider, self).__init__()
-        get = turbogears.config.get
+        glob_ns = globals()
 
-        global user_class, group_class, permission_class, visit_class
-
-        user_class_path = get("identity.soprovider.model.user",
-            __name__ + ".TG_User")
-        user_class = load_class(user_class_path)
-        if user_class:
-            log.info("Successfully loaded \"%s\"" % user_class_path)
+        for classname in ('user', 'group', 'permission', 'visit'):
+            default_classname = '.TG_' + (classname == 'visit'
+                and 'VisitIdentity' or classname.capitalize())
+            class_path = config.get("identity.soprovider.model.%s" % classname,
+                __name__ + default_classname)
+            class_ = load_class(class_path)
+            if class_:
+                log.info('Successfully loaded "%s".', class_path)
+                glob_ns['%s_class' % classname] = class_
+            else:
+                log.error('Could not load class "%s". Check '
+                    'identity.soprovider.model.%s setting', class_path, classname)
         try:
             self.user_class_db_encoding = \
-                user_class.sqlmeta.columns['user_name'].dbEncoding
+                glob_ns['user_class'].sqlmeta.columns['user_name'].dbEncoding
         except (KeyError, AttributeError):
             self.user_class_db_encoding = 'UTF-8'
-        group_class_path = get("identity.soprovider.model.group",
-            __name__ + ".TG_Group")
-        group_class = load_class(group_class_path)
-        if group_class:
-            log.info("Successfully loaded \"%s\"" % group_class_path)
-
-        permission_class_path = get("identity.soprovider.model.permission",
-            __name__ + ".TG_Permission")
-        permission_class = load_class(permission_class_path)
-        if permission_class:
-            log.info("Successfully loaded \"%s\"" % permission_class_path)
-
-        visit_class_path = get("identity.soprovider.model.visit",
-            __name__ + ".TG_VisitIdentity")
-        visit_class = load_class(visit_class_path)
-        if visit_class:
-            log.info("Successfully loaded \"%s\"" % visit_class_path)
 
+    def encrypt_password(self, password):
         # Default encryption algorithm is to use plain text passwords
-        algorithm = get("identity.soprovider.encryption_algorithm", None)
-        self.encrypt_password = lambda pw: \
-            identity._encrypt_password(algorithm, pw)
+        algorithm = config.get("identity.soprovider.encryption_algorithm", None)
+        return identity.encrypt_pw_with_algorithm(algorithm, password)
 
     def create_provider_model(self):
         """Create the database tables if they don't already exist."""
@@ -308,38 +281,33 @@ class SqlObjectIdentityProvider(object):
 
 class TG_VisitIdentity(SQLObject):
     """A visit to your website."""
+
     class sqlmeta:
-        table = "tg_visit_identity"
+        table = "visit_identity"
 
     visit_key = StringCol(length=40, alternateID=True,
         alternateMethodName="by_visit_key")
     user_id = IntCol()
 
 
-class TG_Group(InheritableSQLObject):
+class TG_Group(SQLObject):
     """An ultra-simple group definition."""
-    class sqlmeta:
-        table = "tg_group"
 
     group_name = UnicodeCol(length=16, alternateID=True,
         alternateMethodName="by_group_name")
     display_name = UnicodeCol(length=255)
     created = DateTimeCol(default=datetime.now)
 
-    # Old names
-    groupId = DeprecatedAttr("groupId", "group_name")
-    displayName = DeprecatedAttr("displayName", "display_name")
-
     # collection of all users belonging to this group
-    users = RelatedJoin("TG_User", intermediateTable="tg_user_group",
+    users = RelatedJoin("TG_User", intermediateTable="user_group",
         joinColumn="group_id", otherColumn="user_id")
 
     # collection of all permissions for this group
     permissions = RelatedJoin("TG_Permission", joinColumn="group_id",
-        intermediateTable="tg_group_permission",
+        intermediateTable="group_permission",
         otherColumn="permission_id")
 
-[jsonify.when('isinstance(obj, TG_Group)')]
+@jsonify.when('isinstance(obj, TG_Group)')
 def jsonify_group(obj):
     """Convert group to JSON."""
     result = jsonify_sqlobject(obj)
@@ -348,10 +316,8 @@ def jsonify_group(obj):
     return result
 
 
-class TG_User(InheritableSQLObject):
+class TG_User(SQLObject):
     """Reasonably basic User definition."""
-    class sqlmeta:
-        table = "tg_user"
 
     user_name = UnicodeCol(length=16, alternateID=True,
         alternateMethodName="by_user_name")
@@ -361,13 +327,8 @@ class TG_User(InheritableSQLObject):
     password = UnicodeCol(length=40)
     created = DateTimeCol(default=datetime.now)
 
-    # Old attribute names
-    userId = DeprecatedAttr("userId", "user_name")
-    emailAddress = DeprecatedAttr("emailAddress", "email_address")
-    displayName = DeprecatedAttr("displayName", "display_name")
-
     # groups this user belongs to
-    groups = RelatedJoin("TG_Group", intermediateTable="tg_user_group",
+    groups = RelatedJoin("TG_Group", intermediateTable="user_group",
         joinColumn="user_id", otherColumn="group_id")
 
     def _get_permissions(self):
@@ -395,7 +356,7 @@ class TG_User(InheritableSQLObject):
         """Save the password as-is to the database."""
         self._SO_set_password(password)
 
-[jsonify.when('isinstance(obj, TG_User)')]
+@jsonify.when('isinstance(obj, TG_User)')
 def jsonify_user(obj):
     """Convert user to JSON."""
     result = jsonify_sqlobject(obj)
@@ -405,22 +366,20 @@ def jsonify_user(obj):
     return result
 
 
-class TG_Permission(InheritableSQLObject):
+class TG_Permission(SQLObject):
     """Permissions for a given group."""
+
     class sqlmeta:
-        table = "tg_permission"
+        table = "permission"
 
     permission_name = UnicodeCol(length=16, alternateID=True,
         alternateMethodName="by_permission_name")
     description = UnicodeCol(length=255)
 
-    # Old attributes
-    permissionId = DeprecatedAttr("permissionId", "permission_name")
-
-    groups = RelatedJoin("TG_Group", intermediateTable="tg_group_permission",
+    groups = RelatedJoin("TG_Group", intermediateTable="group_permission",
         joinColumn="permission_id", otherColumn="group_id")
 
-[jsonify.when('isinstance(obj, TG_Permission)')]
+@jsonify.when('isinstance(obj, TG_Permission)')
 def jsonify_permission(obj):
     """Convert permissions to JSON."""
     result = jsonify_sqlobject(obj)
