@@ -7,7 +7,7 @@ import formencode
 import cherrypy
 from turbogears import testutil, database, identity, config, startup, expose
 from turbogears.controllers import Controller, RootController
-from turbogears.identity import (SecureResource, Predicate,
+from turbogears.identity import (SecureObject, SecureResource,
     in_group, in_all_groups, in_any_group, not_anonymous,
     has_permission, has_all_permissions, has_any_permission,
     from_host, from_any_host, Any, All, NotAny)
@@ -108,6 +108,24 @@ class TestPredicates(unittest.TestCase):
         self.assertRaises(RequestRequiredException, _remoteHost)
 
 
+class TestSecureObject(unittest.TestCase):
+
+    def test_secure_object_proxies_cp_attributes(self):
+        """Test that SecureObject properly proxies CherryPy (_cp) attributes."""
+
+        test_filters = ['filter1', 'filter2']
+
+        class TestObject(object):
+            _cp_filters = test_filters
+
+        test_object = SecureObject(TestObject(), None)
+        assert test_object._cp_filters == test_filters
+
+        test_object = SecureObject(RootController(), None)
+        test_object._cp_filters = test_filters
+        assert test_object._cp_filters == test_filters
+
+
 class RestrictedArea(Controller, SecureResource):
 
     require = in_group('peon')
@@ -269,6 +287,10 @@ class IdentityRoot(RootController):
 
 class TestIdentity(testutil.TGTest):
 
+    root = IdentityRoot
+
+    stop_tg_only = True
+
     def setUp(self):
         # identity requires visit and a failure_url
         test_config = {'global': {
@@ -280,20 +302,16 @@ class TestIdentity(testutil.TGTest):
             'identity.http_auth_realm': None,
             'tg.strict_parameters': True,
             'tg.allow_json': False}}
-        original_config = dict()
-        for key in test_config['global']:
-            original_config[key] = config.get(key)
-        self._original_config = original_config
+        if not self.config:
+            original_config = dict()
+            for key in test_config['global']:
+                original_config[key] = config.get(key)
+            self.config = original_config
         config.configure_loggers(test_config)
         config.update(test_config['global'])
-        self.root = IdentityRoot
         super(TestIdentity, self).setUp()
         self.init_model()
 
-    def tearDown(self):
-        super(TestIdentity, self).tearDown()
-        config.update(self._original_config)
-
     def init_model(self):
         if TG_User.select().count() == 0:
             user = TG_User(user_name='samIam',
@@ -628,7 +646,7 @@ class TestIdentity(testutil.TGTest):
     def test_user_unicode(self):
         """Test that we can have non-ascii user names."""
         user = TG_User(user_name=u'säm', display_name=u'Sämüel Käsfuß',
-            email_address='sam@example.com', password='geheim')
+            email_address='samkas@example.com', password='geheim')
         response = self.app.get('/logged_in_only?'
             'user_name=säm&password=geheim&login=Login')
         assert 'logged_in_only' in response, response
@@ -648,6 +666,7 @@ class TestIdentity(testutil.TGTest):
         response = self.app.get('/json?tg_format=json', status=403)
         # we get the right status code, but not the output of identity_failure
         assert 'identity_failed_answer' not in response, response
+        assert 'Request forbidden' in response, response
         response = self.app.get('/peon_area/json', status=403)
         assert 'identity_failed_answer' in response, response
         response = self.app.get('/peon_area/json?tg_format=json', status=403)
@@ -664,10 +683,8 @@ class TestIdentity(testutil.TGTest):
         response = self.app.get('/peon_area/json?tg_format=json', status=200)
         assert 'restricted_json' in response, response
 
-
     def test_remote_ip(self):
         """Test that our client IP is detected correctly."""
-        r = self.app.get('/remote_ip', status=200)
         r = self.app.get('/remote_ip', headers={'Remote-Addr': '127.0.0.1'},
             status=200)
         assert r.raw['remote_ip'] == '127.0.0.1'
@@ -697,15 +714,28 @@ class TestIdentity(testutil.TGTest):
             status=200)
         assert 'hosts_on_list_only' in r
 
+    def test_nested_login(self):
+        """Check that we can login using a nested form."""
+        config.update({
+            'identity.form.user_name': 'credentials.user',
+            'identity.form.password': 'credentials.pass',
+            'identity.form.submit': 'log.me.in'})
+        testutil.stop_server(tg_only=False)
+        self.app = testutil.make_app(self.root)
+        testutil.start_server()
+        response = self.app.get('/logged_in_only?'
+            'credentials.user=samIam&credentials.pass=secret&log.me.in=Enter')
+        assert 'logged_in_only' in response, response
+
 
 class TestTGUser(testutil.DBTest):
+
     model = TG_User
 
     def setUp(self):
         self._identity_on = config.get('identity.on', False)
         config.update({'identity.on': False})
         super(TestTGUser, self).setUp()
-        testutil.start_server()
 
     def tearDown(self):
         testutil.stop_server()
@@ -721,19 +751,23 @@ class TestTGUser(testutil.DBTest):
 
 
 class TestIdentityVisitPlugin(unittest.TestCase):
+
+    config = None
+
     def setUp(self):
         test_config = {'global': {
             'visit.on': True,
             'identity.on': True,
-            'identity.source': 'form, http_auth, visit, '}}
-        original_config = dict()
-        for key in test_config['global']:
-            original_config[key] = config.get(key)
-        self._original_config = original_config
+            'identity.source': 'form, http_auth, visit'}}
+        if not self.config:
+            original_config = dict()
+            for key in test_config['global']:
+                original_config[key] = config.get(key)
+            self.config = original_config
         config.update(test_config['global'])
 
     def tearDown(self):
-        config.update(self._original_config)
+        config.update(self.config)
 
     def test_identity_source(self):
         """Test that identity.source setting is parsed correctly."""
