@@ -57,7 +57,7 @@ def paginate(var_name, default_order='', limit=10,
     """The famous TurboGears paginate decorator.
 
     @param var_name: the variable name that the paginate decorator will try
-    to control. This key must be present in the dictionnary returned from your
+    to control. This key must be present in the dictionary returned from your
     controller in order for the paginate decorator to be able to handle it.
     @type var_name: string
 
@@ -394,26 +394,29 @@ def sort_ordering(ordering, sort_name):
         del ordering[index]
         ordering.insert(0, (not index and '-' or '') + sort_name)
 
-def sqlalchemy_get_column(colname, var_data):
+def sqlalchemy_get_column(colname, var_data, join_props=None):
     """Return a column from sqlalchemy var_data based on colname."""
     try:
         mapper = var_data.mapper
-    except AttributeError:
-        try:
-            mapper = var_data._mapper_zero()
-        except AttributeError:
-            mapper = var_data._query.mapper
+    except AttributeError: # SQLALchemy >= 0.5
+        mapper = var_data._mapper_zero()
     propnames = colname.split('.')
-    colname = propnames.pop()[len(mapper.column_prefix or ''):]
+    colname = propnames.pop()
+    props = []
     for propname in propnames:
         prop = mapper.get_property(
             propname, resolve_synonyms=True, raiseerr=False)
         if not prop:
             break
+        if join_props is not None:
+            props.append(prop)
         mapper = prop.mapper
-    return getattr(mapper.c, colname, None)
+    col = getattr(mapper.c, colname, None)
+    if col is not None and props:
+        join_props.extend(props)
+    return col
 
-def sqlobject_get_column(colname, var_data):
+def sqlobject_get_column(colname, var_data, join_props=None):
     """Return a column from sqlobject var_data based on colname."""
     return getattr(var_data.sourceClass.q, colname, None)
 
@@ -454,6 +457,23 @@ def sql_order_col(col, ascending=True):
         return sqlobject_order_col(col, ascending)
     raise TypeError("Expected Column, but got %s" % type(col))
 
+def sqlalchemy_join_props(var_data, join_props):
+    """Return a query where all tables for properties are joined."""
+    if join_props:
+        attrs = []
+        for prop in join_props:
+            try:
+                attrs.append(prop.class_attribute)
+            except AttributeError: # SQLAlchemy < 0.6
+                try:
+                    attrs.append(getattr(prop.parent.class_, prop.key))
+                except AttributeError:
+                    pass
+        if attrs:
+            log.debug("paginate: need to join some attributes")
+            var_data = var_data.outerjoin(*attrs)
+    return var_data
+
 def sort_data(data, ordering, in_memory=True):
     """Sort data based on ordering.
 
@@ -462,17 +482,22 @@ def sort_data(data, ordering, in_memory=True):
 
     """
     try:
-        order_by = data.order_by # SQLAlchemy
+        order_by = data.order_by # SQLAlchemy, gets variable argument list
+        if hasattr(data, 'get_by'): # SQLAlchemy < 0.5
+            old_order_by = order_by # old version, gets single argument
+            order_by = lambda *cols: old_order_by(cols)
         get_column, order_col = sqlalchemy_get_column, sqlalchemy_order_col
     except AttributeError:
         try:
-            order_by = data.orderBy # SQLObject
+            orderBy = data.orderBy  # SQLObject, gets single argument
+            order_by = lambda *cols: orderBy(cols)
             get_column, order_col = sqlobject_get_column, sqlobject_order_col
         except AttributeError:
             order_by = None
     order_cols = []
     key_cols = []
     num_ascending = num_descending = 0
+    join_props = []
     for order in ordering:
         if order[0] == '-':
             order = order[1:]
@@ -480,8 +505,8 @@ def sort_data(data, ordering, in_memory=True):
         else:
             descending = False
         if order_by:
-            col = get_column(order, data)
-            if col:
+            col = get_column(order, data, join_props)
+            if col is not None:
                 order_cols.append(order_col(col, descending))
                 continue
         if not order_cols:
@@ -491,7 +516,9 @@ def sort_data(data, ordering, in_memory=True):
             else:
                 num_ascending += 1
     if order_by and order_cols:
-        data = order_by(order_cols)
+        data = order_by(*order_cols)
+        if join_props:
+            data = sqlalchemy_join_props(data, join_props)
     if key_cols:
         if in_memory:
             data = list(data)
@@ -524,6 +551,7 @@ def sort_data(data, ordering, in_memory=True):
                             return lambda row: keys[key(row)]
                 keys = [reverse_key(*key) for key in keys]
                 key = lambda row: [key(row) for key in keys]
+            log.debug("paginate: sorting in memory")
             data.sort(key=key, reverse=reverse)
         else:
             log.debug("paginate: sorting in memory not allowed")
