@@ -32,8 +32,8 @@ pkg_resources.require("TurboGears")
 
 from turbogears import config, database, scheduler, view
 from turbogears.database import hub_registry, EndTransactionsFilter
-from turbogears.filters import (NestedVariablesFilter, SafeMultipartFilter,
-    VirtualPathFilter)
+from turbogears.filters import (MonkeyDecodingFilter, NestedVariablesFilter,
+    SafeMultipartFilter, VirtualPathFilter)
 
 
 # module globals
@@ -167,10 +167,10 @@ def startTurboGears():
     * If logging is not already set up, turns on old-style stdlib logging.
     * Adds a static filter for TurboGears's static files (URL '/tg_static').
     * Adds a static filter for TurboGears's JavaScript files (URL '/tg_js').
-    * Adds the decoding filter to the root URL ('/') if enabled in the
-      configuration.
     * Loads the template engines and the base templates.
     * Adds the CherryPy request filters to the root controller.
+    * Adds the decoding filter to the root URL ('/') if enabled in the
+      configuration.
     * Registers the server with the Bonjour framework, if available.
     * Calls 'turbogears.database.bind_metadata' when using SQLAlchemy.
     * Loads all turbogears.extensions entry points and calls their
@@ -188,7 +188,7 @@ def startTurboGears():
 
     # XXX: obsolete --> to be removed
     # Set up old-style logging
-    if not conf('tg.new_style_logging'):
+    if not conf('tg.new_style_logging', False):
         if conf('server.log_to_screen'):
             setuplog = logging.getLogger()
             setuplog.setLevel(logging.DEBUG)
@@ -251,6 +251,20 @@ def startTurboGears():
                     break
         cherrypy.root._cp_filters.extend(extra_filters)
 
+    # Monkey patch CherryPy Decoding filter: injects our replacement filter
+    # MonkeyDecodingFilter into the CherryPy filter chain
+    decoding_filter = MonkeyDecodingFilter()
+    for index, active_filter in enumerate(
+            cherrypy.filters._filterhooks.get('before_main', [])):
+        if active_filter.im_class == \
+                cherrypy.filters.decodingfilter.DecodingFilter:
+            cherrypy.filters._filterhooks['before_main'].pop(index)
+            if conf('decoding_filter.on', False, path='/'):
+                log.info("Request decoding filter activated.")
+                cherrypy.filters._filterhooks['before_main'].insert(
+                        index, decoding_filter.before_main)
+
+
     webpath = webpath.lstrip('/')
     if webpath and not webpath.endswith('/'):
         webpath += '/'
@@ -313,9 +327,19 @@ def stopTurboGears():
     # Shut down all TurboGears extensions
     extensions= pkg_resources.iter_entry_points( "turbogears.extensions" )
     for entrypoint in extensions:
-        ext = entrypoint.load()
+        try:
+            ext = entrypoint.load()
+        except Exception, e:
+            log.exception("Error loading TurboGears extension plugin '%s': %s",
+                entrypoint, e)
+            continue
         if hasattr(ext, "shutdown_extension"):
-            ext.shutdown_extension()
+            try:
+                ext.shutdown_extension()
+            except Exception, e:
+                log.exception(
+                    "Error shutting down TurboGears extension '%s': %s",
+                    entrypoint, e)
 
     for item in call_on_shutdown:
         item()
