@@ -1,6 +1,7 @@
 
 import sys
 from StringIO import StringIO
+from warnings import simplefilter, resetwarnings
 
 from turbogears import config, util
 
@@ -12,25 +13,30 @@ def test_deprecated():
         """some old function"""
         return x + y
 
-    assert "some old function" in some_old_function.__doc__
-    some_old_function.__name__ == 'some_old_function'
+    assert 'some old function' in some_old_function.__doc__
+    assert some_old_function.__name__ == 'some_old_function'
 
     stderr, sys.stderr = sys.stderr, StringIO()
     try:
-        assert some_old_function(1, 2) == 3
+        result = some_old_function(1, 2)
     finally:
         stderr, sys.stderr = sys.stderr, stderr
-    assert 'DeprecationWarning: this is old' in stderr.getvalue()
+    assert result == 3, result
+    output = stderr.getvalue()
     stderr.close()
+    assert 'DeprecationWarning: this is old' in output, output
 
     # Deprecation warnings should only be emitted once.
     stderr, sys.stderr = sys.stderr, StringIO()
     try:
-        assert some_old_function(2, 3) == 5
+        result = some_old_function(2, 3)
     finally:
         stderr, sys.stderr = sys.stderr, stderr
-    assert 'DeprecationWarning: this is old' not in stderr.getvalue()
+    assert result == 5, result
+    output = stderr.getvalue()
     stderr.close()
+    assert 'DeprecationWarning' not in output, output
+    assert 'this is old' not in output, output
 
 
 def test_adapt_call():
@@ -152,6 +158,43 @@ def test_arg_index():
     assert util.arg_index(fun, 'z') == 3
 
 
+def test_pop_from_nested_dict():
+    pop = util.pop_from_nested_dict
+    params = {'likes': {
+     'fruits': {'bananas': 1, 'apples': 2, 'peaches': 3},
+     'languages': {'python': 4, 'c#': 5, 'java': 6, 'fortan': 7}},
+     'dislikes': {'nested': {'structures': 8}},
+     'submit': {None: 9, 'x': 10, 'y': 11},
+     'more': {'submit': {None: 12}},
+     'user_name': 'homer', 'password': 'doh'
+     }
+    assert pop(params, 'user_name') == 'homer'
+    assert 'user_name' not in params
+    assert pop(params, ['password']) == 'doh'
+    assert 'password' not in params
+    assert pop(params, 'fruits') is None
+    assert pop(params, 'likes.fruits') is None
+    assert pop(params, 'likes.fruits.apples') == 2
+    assert 'likes' in params
+    assert 'fruits' in params['likes']
+    assert 'apples' not in params['likes']['fruits']
+    assert 'peaches' in params['likes']['fruits']
+    assert pop(params, 'likes.fruits.bananas') == 1
+    assert pop(params, ['likes', 'fruits', 'peaches']) == 3
+    assert 'likes' in params
+    assert 'fruits' not in params['likes']
+    assert pop(params, 'dislikes.nested') is None
+    assert pop(params, 'dislikes.nested.structures') == 8
+    assert 'dislikes' not in params
+    assert pop(params, 'submit') == 9
+    assert 'submit' in params
+    assert params['submit'] == {'x': 10, 'y': 11}
+    assert pop(params, 'more') is None
+    assert pop(params, 'more.submit.buttons') is None
+    assert pop(params, 'more.submit') == 12
+    assert 'more' not in params
+
+
 def test_parse_http_accept_header():
     t = util.parse_http_accept_header
     assert t(None) == []
@@ -207,17 +250,21 @@ def test_preferred_from_http_accept_header():
 
 
 def test_quote_cookie():
-    assert util.quote_cookie ('Hello, W\xf6rld') \
-        == 'Hello%2C%20W\xf6rld'
-    assert util.quote_cookie ('$1;\tinsert coin!') \
-        == '%241%3B%09insert%20coin!'
+    quote = util.quote_cookie
+    assert quote('Hello!') == 'Hello!'
+    assert quote('K\xe4se') == '"K\\344se"'
+    assert quote('Hello, W\xf6rld!') == '"Hello, W\\366rld!"'
+    assert quote('$1;\tinsert coin!') == '"$1;\\011insert coin!"'
+    assert quote('Hello, "W\\orld"!') == r'"Hello, \"W\\orld\"!"'
 
 
 def test_unquote_cookie():
-    assert util.unquote_cookie ('Hello%2C%20W\xf6rld!') \
-        == 'Hello, W\xf6rld!'
-    assert util.unquote_cookie ('%241%3B%09insert%20coin!') \
-        == '$1;\tinsert coin!'
+    unquote = util.unquote_cookie
+    assert unquote('Hello!') == 'Hello!'
+    assert unquote('"K\\344se"') == 'K\xe4se'
+    assert unquote('"Hello, W\\366rld!"') == 'Hello, W\xf6rld!'
+    assert unquote('"$1;\\011insert coin!"') == '$1;\tinsert coin!'
+    assert unquote(r'"Hello, \"W\\orld\"!"') == 'Hello, "W\\orld"!'
 
 
 def test_get_template_encoding_default():
