@@ -142,7 +142,7 @@ class TestKidView(unittest.TestCase):
         ustr = u"micro-eXtreme Programming ( µ XP): Embedding XP Within Standard Projects"
         info = dict(someval=ustr)
         val = view.render(info, template="kid:turbogears.tests.simple")
-        self.failUnless(u"Paging all " + ustr in val.decode("utf-8"))
+        self.assertTrue(u"Paging all " + ustr in val.decode("utf-8"))
 
     def test_templateRetrievalByPath(self):
         """Test that kid can find, load and compile templates."""
@@ -354,11 +354,16 @@ class TestGenshiView(unittest.TestCase):
             info = dict(someval="dumbo")
             headers = {}
             val = view.render(info, template, headers=headers)
+            assert val.lstrip().startswith('<!DOCTYPE')
             assert html_strict_url in val
             val = view.render(info, template, headers=headers, format='xhtml')
+            assert val.lstrip().startswith('<!DOCTYPE')
             assert xhtml_strict_url in val
+            val = view.render(info, template, headers=headers, fragment=True, format='xhtml')
+            assert not val.lstrip().startswith('<!DOCTYPE')
+            assert xhtml_strict_url not in val
             val = view.render(info, template, headers=headers, format='xml')
-            assert not val.strip().startswith('<!DOCTYPE')
+            assert not val.lstrip().startswith('<!DOCTYPE')
 
         def test_genshi_set_doctypes(self):
             """Test that we can set the doctpye Genshi uses for each rendering.
@@ -367,6 +372,7 @@ class TestGenshiView(unittest.TestCase):
             info = dict(someval="dumbo")
             headers = {}
             val = view.render(info, template, headers=headers, doctype='html')
+            assert val.lstrip().startswith('<!DOCTYPE')
             assert html_strict_url in val
             val = view.render(info, template, headers=headers,
                               doctype='html-strict')
@@ -381,7 +387,12 @@ class TestGenshiView(unittest.TestCase):
             assert html_strict_url in val
             val = view.render(info, template, headers=headers, format='xhtml',
                               doctype='xhtml-strict')
+            assert val.lstrip().startswith('<!DOCTYPE')
             assert xhtml_strict_url in val
+            val = view.render(info, template, headers=headers, format='xhtml',
+                              fragment=True, doctype='xhtml-strict')
+            assert not val.lstrip().startswith('<!DOCTYPE')
+            assert xhtml_strict_url not in val
             val = view.render(info, template, headers=headers, format='xhtml',
                               doctype='xhtml-transitional')
             assert xhtml_transitional_url in val
