@@ -8,12 +8,12 @@ from turbogears.database import PackageHub
 from turbogears.util import load_class
 from turbogears.visit.api import BaseVisitManager, Visit
 
-hub = PackageHub("turbogears.visit")
+hub = PackageHub('turbogears.visit')
 __connection__ = hub
 
 import logging
 
-log = logging.getLogger("turbogears.visit.sovisit")
+log = logging.getLogger('turbogears.visit.sovisit')
 
 visit_class = None
 
@@ -22,11 +22,11 @@ class SqlObjectVisitManager(BaseVisitManager):
 
     def __init__(self, timeout):
         global visit_class
-        visit_class_path = config.get("visit.soprovider.model",
-            "turbogears.visit.sovisit.TG_Visit")
+        visit_class_path = config.get('visit.soprovider.model',
+            'turbogears.visit.sovisit.TG_Visit')
         visit_class = load_class(visit_class_path)
         if visit_class:
-            log.info('Succesfully loaded "%s"', visit_class_path)
+            log.info("Successfully loaded '%s'", visit_class_path)
         # base-class' __init__ triggers self.create_model, so mappers need to
         # be initialized before.
         super(SqlObjectVisitManager, self).__init__(timeout)
@@ -67,8 +67,8 @@ class SqlObjectVisitManager(BaseVisitManager):
         return Visit(visit_key, False)
 
     def update_queued_visits(self, queue):
-        if hub is None: # if VisitManager extension wasn't shutted down cleanly
-            return
+        if hub is None:
+            return # if VisitManager extension wasn't shutted down cleanly
         hub.begin()
         try:
             conn = hub.getConnection()
@@ -77,7 +77,7 @@ class SqlObjectVisitManager(BaseVisitManager):
                 for visit_key, expiry in queue.items():
                     u = Update(visit_class.q,
                         {visit_class.q.expiry.fieldName: expiry},
-                        where=(visit_class.q.visit_key==visit_key))
+                        where=(visit_class.q.visit_key == visit_key))
                     conn.query(conn.sqlrepr(u))
                 hub.commit()
             except:
@@ -90,10 +90,10 @@ class SqlObjectVisitManager(BaseVisitManager):
 class TG_Visit(SQLObject):
 
     class sqlmeta:
-        table = "tg_visit"
+        table = 'visit'
 
     visit_key = StringCol(length=40, alternateID=True,
-            alternateMethodName="by_visit_key")
+            alternateMethodName='by_visit_key')
     created = DateTimeCol(default=datetime.now)
     expiry = DateTimeCol()
 
