@@ -1,15 +1,18 @@
-import os
 import itertools
-import pkg_resources
+import os
 import warnings
-from turbogears import view, startup, config
-from turbogears.util import setlike, to_unicode, copy_if_mutable, \
-    get_package_name, request_available
-from turbogears.i18n.utils import get_locale
-from turbogears.widgets.meta import MetaWidget, load_kid_template
+
+import pkg_resources
+
 from cherrypy import request
 import tgmochikit
 
+from turbogears import config, startup, view
+from turbogears.util import (setlike, to_unicode, copy_if_mutable,
+    get_package_name, request_available)
+from turbogears.i18n.utils import get_locale
+from turbogears.widgets.meta import MetaWidget, load_kid_template
+
 __all__ = ['load_widgets', 'all_widgets', 'Widget', 'CompoundWidget',
     'WidgetsList', 'register_static_directory', 'static',
     'Resource', 'Link', 'CSSLink', 'JSLink',
@@ -79,8 +82,8 @@ PlainHTML = load_kid_template("""
 class Widget(object):
     """A TurboGears Widget.
 
-   '__init__' and 'update_params' are the only methods you might need to
-   care extending.
+    '__init__' and 'update_params' are the only methods you might need to
+    care extending.
 
     Attributes you should know about:
 
@@ -103,28 +106,27 @@ class Widget(object):
     params_doc: A dictionary containing 'params' names as keys and their
                 docstring as value. For documentation at the widget browser.
 
-   All initialization parameters listed at the class attribute "params" can be
-   defined as class attributes, overriden at __init__ or at display time. They
-   will be treated as special params for the widget, which means:
+    All initialization parameters listed at the class attribute "params" can be
+    defined as class attributes, overriden at __init__ or at display time. They
+    will be treated as special params for the widget, which means:
 
-   1) You can fix default values for them when sublcassing Widget.
+    1) You can fix default values for them when sublcassing Widget.
 
-   2) If passed as **params to the constructor, the will be bound automatically
-      to the widget instance, taking preference over any class attributes
-      previously defined. Mutable attributes (dicts and lists) defined as class
-      attributes are safe to modify as care is taken to copy them so the class
-      attribute remains unchanged.
+    2) If passed as **params to the constructor, the will be bound automatically
+       to the widget instance, taking preference over any class attributes
+       previously defined. Mutable attributes (dicts and lists) defined as class
+       attributes are safe to modify as care is taken to copy them so the class
+       attribute remains unchanged.
 
-   3) They can be further overrided by passing them as keyword args to
-      display(). This will only affect that display() call in a thread-safe
-      way.
+    3) They can be further overrided by passing them as keyword args to
+       display(). This will only affect that display() call in a thread-safe
+       way.
 
-   4) A callable can be passed and it will be called automatically when sending
-      variables to the template. This can be handy to pick up parameters which
-      change in every request or affect many widgets simultaneously.
+    4) A callable can be passed and it will be called automatically when sending
+       variables to the template. This can be handy to pick up parameters which
+       change in every request or affect many widgets simultaneously.
 
     """
-
     __metaclass__ = MetaWidget
 
     name = "widget"
@@ -185,15 +187,12 @@ class Widget(object):
                 setattr(self, param, copy_if_mutable(params.pop(param)))
 
             else:
-                if hasattr(self, param):
-                    # make sure we don't alter mutable class attributes
-                    value = copy_if_mutable(getattr(self.__class__, param),
-                                            True)
-                    if value[1]:
-                        # re-set it only if mutable
-                        setattr(self, param, value[0])
-                else:
-                    setattr(self, param, None)
+                # make sure we don't alter mutable class attributes
+                value, mutable = copy_if_mutable(
+                    getattr(self.__class__, param), True)
+                if mutable:
+                    # re-set it only if mutable
+                    setattr(self, param, value)
 
         for unused in params.iterkeys():
             warnings.warn('keyword argument "%s" is unused at %r instance' % (
@@ -201,10 +200,6 @@ class Widget(object):
 
     def adjust_value(self, value, **params):
         """Adjust the value sent to the template on display."""
-        if value is None:
-            value = self.default
-            if callable(value):
-                value = value()
         return value
 
     def update_params(self, params):
@@ -236,7 +231,7 @@ class Widget(object):
         Returns an elementtree node instance. If you need serialized output
         in a string, call 'render' instead.
 
-        Probably you will not need to override or extend if inhertiting from
+        Probably you will not need to override or extend if inheriting from
         Widget.
 
         @params:
@@ -248,9 +243,11 @@ class Widget(object):
                   template.
 
         """
-
         if not getattr(self, 'template_c', False):
-            warnings.warn("Widget instance '%r' has no template defined" % self)
+            warnings.warn('kid' in view.engines
+                and "Widget instance %r has no template defined" % self
+                or "Trying to render a widget, but the Kid"
+                    " templating engine is not installed or not yet loaded.")
             return None
 
         # logic for managing the params attribute
@@ -271,23 +268,43 @@ class Widget(object):
 
         if not params.get('name'):
             params['name'] = self.name
+
+        if value is None:
+            value = self.default
+            if callable(value):
+                value = value()
         params['value'] = to_unicode(self.adjust_value(value, **params))
+
         self.update_params(params)
 
         try:
             transform = view.engines['kid'].transform
-
         except (KeyError, AttributeError):
             # this can happen if you render a widget before application startup
             # when view.load_engines() has not yet been called
-            raise RuntimeError("Trying to render a widget,"
-                " but Kid templating engine is not yet loaded.")
+            raise RuntimeError("Trying to render a widget, but the Kid"
+                " templating engine is not installed or not yet loaded.")
+
+        # If the page template is Genshi, we keep track of the nesting level,
+        # because Genshi cannot display Kid's ElementTree elements directly.
+        if request_available() and request.tg_template_enginename == 'genshi':
+            display_level = getattr(request, 'tg_widgets_display_level', 0)
+            request.tg_widgets_display_level = display_level + 1
+        else:
+            display_level = None
+
+        try:
+            output = transform(params, self.template_c)
+            if display_level == 0:
+                # On the topmost level, we create a Genshi markup stream
+                # from Kid's ElementTree element to make Genshi really happy.
+                # This automatism makes wrapping widget output with ET(...)
+                # calls in Genshi page templates unnecessary.
+                output = view.genshi_et(output)
+        finally:
+            if display_level is not None:
+                request.tg_widgets_display_level = display_level
 
-        output = transform(params, self.template_c)
-        if request_available() and request.tg_template_enginename == 'genshi' \
-                and len(getattr(request, 'tg_widgets_path', [])) == 1:
-            import genshi
-            output = genshi.input.ET(output)
         return output
 
     def render(self, value=None, format="html", **params):
@@ -298,8 +315,8 @@ class Widget(object):
 
         """
         elem = self.display(value, **params)
-        t = PlainHTML(elements=elem)
-        return t.serialize(output=format, fragment=True)
+        template = PlainHTML(elements=elem)
+        return template.serialize(output=format, fragment=True)
 
     def retrieve_javascript(self):
         """Return the needed JavaScript ressources.
@@ -468,7 +485,8 @@ class MetaWidgetsList(type):
                 if not value.is_named:
                     value.name = name
                 declared_widgets.append(value)
-                del class_dict[name]
+                # we keep the widget accessible as attribute here,
+                # i.e. we don't del class_dict[name] as we did before
         declared_widgets.sort(key=lambda w: w._declaration_counter)
         cls = type.__new__(meta, class_name, bases, class_dict)
         cls.declared_widgets = declared_widgets
@@ -573,7 +591,7 @@ class Link(Resource):
 
 
 class CSSLink(Link):
-    """A CSS links."""
+    """A CSS link."""
 
     template = """
     <link rel="stylesheet"
@@ -623,11 +641,17 @@ class TGMochiKit(JSLink):
 
     Depending on three config options, you can decide
 
-    - tg_mochikit.version (default: 1.3.1) - which version to use
-    - tg_mochikit.packed  (default: False) - deliver the JS packed or unpacked
-    - tg_mochikit.xhtml   (default: False) - be XHTML-compliant or not in case you deliver unpacked
+    ``tg_mochikit.version`` -- ``"1.3.1"``
+        Which version of MochiKit should be used.
+    ``tg_mochikit.packed`` -- ``False``
+        Whether to deliver the MochiKit JS code packed or unpacked.
+    ``tg_mochikit.xhtml`` -- ``False``
+        Whether to be XHTML-compliant or not when ``tg_mochikit.packed`` is
+        ``False``, i.e. selects whether the ``turbogears.mochikit`` widget
+        should include all of MochiKit's submodules as separate JavaScript
+        resources or just the main ``MochiKit.js`` file.
 
-    For an explanation of the options, see the tgMochiKit documentation at
+    For more explanation of the options, see the tgMochiKit documentation at
 
         http://docs.turbogears.org/tgMochiKit
 
@@ -678,7 +702,6 @@ class JSI18NWidget(Widget):
     in your application configuration.
 
     """
-
     params = ['locale_catalog_providers']
 
     locale_catalog_providers = []
@@ -740,14 +763,12 @@ class JSI18NWidget(Widget):
         return js
 
     def linked_file_exists(self, widget):
-        key = '/tg_widgets/%s' % widget.mod
-        wp = config.get(key)
-        if wp:
-            dir = wp.get('static_filter.dir')
-            if dir:
-                path = os.path.join(dir, widget.name)
-                return os.path.exists(path)
-        return False
+        static_dir = config.get('static_filter.dir',
+            path='/tg_widgets/%s' % widget.mod)
+        if static_dir:
+            return os.path.exists(os.path.join(static_dir, widget.name))
+        else:
+            return False
 
 jsi18nwidget = JSI18NWidget()
 
